@@ -1,3 +1,5 @@
+import platform
+
 try:
     from importlib.metadata import version as _version, PackageNotFoundError
 
@@ -37,3 +39,5 @@ VERSION_WITH_DEPS = (
     + ", ".join(f"{n} {dep_version(n)}" for n in ("gpg-lite", "libbiomedit"))
     + ")"
 )
+
+RUNTIME_INFO = f"{VERSION_WITH_DEPS} (platform: {platform.platform()})"
