@@ -6,8 +6,7 @@ from typing import Tuple, Iterable, IO
 from .error import UserError
 
 
-def compute_checksum_sha256(file_object: IO[bytes],
-                            block_size: int = 65536) -> str:
+def compute_checksum_sha256(file_object: IO[bytes], block_size: int = 65536) -> str:
     """Compute a sha256 checksum on a file by reading it by chunks of
     'block_size' bytes at a time.
 
@@ -24,10 +23,12 @@ def compute_checksum_sha256(file_object: IO[bytes],
     return hash_value.hexdigest()
 
 
-def compute_checksum_on_write(stream: IO[bytes],
-                              fout: io.FileIO,
-                              checksum_buffer: io.StringIO,
-                              block_size: int = 65536) -> None:
+def compute_checksum_on_write(
+    stream: IO[bytes],
+    fout: io.FileIO,
+    checksum_buffer: io.StringIO,
+    block_size: int = 65536,
+) -> None:
     """Compute a sha256 checksum on stream while writing it to a file at the
     same time.
 
@@ -67,17 +68,15 @@ def write_checksums(entries: Iterable[Tuple[str, IO[bytes]]]) -> bytes:
     for archive_path, opener in entries:
         with opener as f_hash:
             checksum = compute_checksum_sha256(file_object=f_hash)
-        if os.path.sep == '/' and '\\' in archive_path:
+        if os.path.sep == "/" and "\\" in archive_path:
             raise UserError("On POSIX systems, backslashes are NOT allowed.")
         # Replace DOS path separators with Unix ones
         posix_path = Path(archive_path).as_posix()
-        lines.append((
-            f'{checksum} {posix_path}\n').encode())
+        lines.append((f"{checksum} {posix_path}\n").encode())
     return b"".join(lines)
 
 
-def verify_checksums(checksums: Iterable[Tuple[str, str]],
-                     base_path: str) -> None:
+def verify_checksums(checksums: Iterable[Tuple[str, str]], base_path: str) -> None:
     """Checks that the checksum values of files listed in the 'checksums'
     generator match their hash values (also provided in 'checksums'). The
     function raises an error if a mismatch is detected.
@@ -89,14 +88,15 @@ def verify_checksums(checksums: Iterable[Tuple[str, str]],
     for checksum, file_name in checksums:
         # Open file in read-only mode and verify its checksum is the same
         # as the one given in input_file. File name should be posix, just in case it is NOT...
-        file_path = Path(base_path) / \
-            Path(file_name.replace('\\', os.path.sep))
-        with open(file_path, 'rb') as f:
+        file_path = Path(base_path) / Path(file_name.replace("\\", os.path.sep))
+        with open(file_path, "rb") as f:
             computed = compute_checksum_sha256(f)
             checksum = checksum.lower()
             if checksum != computed:
-                raise UserError(f'Checksum mismatch for: {file_name}\nExpected: {checksum}'
-                                f'\nGot: {computed}')
+                raise UserError(
+                    f"Checksum mismatch for: {file_name}\nExpected: {checksum}"
+                    f"\nGot: {computed}"
+                )
 
 
 def read_checksum_file(f: IO[bytes]) -> Iterable[Tuple[str, str]]:
