@@ -1,20 +1,24 @@
 #!/usr/bin/env python3
 """Secure Encryption and Transfer Tool."""
 
-import sys
 import logging
+import sys
 
+from sett.gui.component import is_macos, MACOS_TOOLTIP_BG_COLOR
 from . import main_window
-from .pyside import QtWidgets
-from ..utils.log import add_rotating_file_handler_to_logger
+from .pyside import QtWidgets, QtGui
+from .resources import rc_icons
+from ..utils.log import log_to_rotating_file
 
 
 def run():
     application = QtWidgets.QApplication(sys.argv)
+    if is_macos():
+        palette = application.palette()
+        palette.setColor(QtGui.QPalette.ColorRole.ToolTipBase, MACOS_TOOLTIP_BG_COLOR)
+        application.setPalette(palette)
     window = main_window.MainWindow()
-    logger = logging.getLogger()
-    add_rotating_file_handler_to_logger(
-        logger,
+    log_to_rotating_file(
         log_dir=window.app_data.config.log_dir,
         file_max_number=window.app_data.config.log_max_file_number,
     )
