@@ -10,6 +10,7 @@ from .component import (
     SelectionButton,
     MandatoryLabel,
     create_verify_dtr_checkbox,
+    LineEdit,
 )
 from .file_selection_widget import DirectoryAndFileSelectionWidget
 from .model import TableModel
@@ -23,7 +24,6 @@ from ..workflows import encrypt
 class EncryptTab(QtWidgets.QWidget, TabMixin):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.setLayout(QtWidgets.QVBoxLayout(self))
 
         self.threadpool = QtCore.QThreadPool.globalInstance()
         self.app_data = self.parent().app_data
@@ -37,11 +37,12 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         self.create_console()
         self.create_progress_bar()
 
-        self.layout().addWidget(files_panel)
-        self.layout().addLayout(self._create_middle_layout())
-        self.layout().addWidget(self.run_panel)
-        self.layout().addWidget(self.console)
-        self.layout().addWidget(self.progress_bar)
+        layout = QtWidgets.QVBoxLayout(self)
+        layout.addWidget(files_panel)
+        layout.addLayout(self._create_middle_layout())
+        layout.addWidget(self.run_panel)
+        layout.addWidget(self.console)
+        layout.addWidget(self.progress_bar)
 
     def _enable_buttons(self):
         self.set_buttons_enabled(
@@ -82,11 +83,15 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
             QtWidgets.QTableView.SelectionMode.SingleSelection
         )
 
-        recipients_btn_add = QtWidgets.QPushButton("+")
-        recipients_btn_add.setStatusTip("Add recipient to the list")
-        recipients_btn_remove = SelectionButton(
-            "Remove selected recipients", recipients_output_view.selectionModel()
+        recipients_btn_add = QtWidgets.QPushButton(
+            QtGui.QIcon(":icon/feather/user-plus.png"), ""
         )
+        recipients_btn_add.setToolTip("Add recipient to the list")
+        recipients_btn_remove = SelectionButton(
+            "", recipients_output_view.selectionModel()
+        )
+        recipients_btn_remove.setIcon(QtGui.QIcon(":icon/feather/user-minus.png"))
+        recipients_btn_remove.setToolTip("Remove recipient from the list")
 
         def update_sender(index):
             self.app_data.encrypt_sender = (
@@ -134,26 +139,26 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         layout.addWidget(recipients_input_view, 2, 1, 1, 1)
         layout.addWidget(recipients_btn_add, 2, 2, 1, 1)
         layout.addWidget(MandatoryLabel("Recipients"), 3, 0, 1, 1, QtCore.Qt.AlignTop)
-        layout.addWidget(recipients_output_view, 3, 1, 1, 2)
-        layout.addWidget(recipients_btn_remove, 4, 0, 1, 3)
+        layout.addWidget(recipients_output_view, 3, 1, 1, 1)
+        layout.addWidget(
+            recipients_btn_remove, 3, 2, 1, 1, alignment=QtCore.Qt.AlignTop
+        )
 
-        group_box.setLayout(layout)
         return group_box
 
     def _create_middle_layout(self):
-        grid_layout = QtWidgets.QGridLayout()
-        grid_layout.addWidget(self.create_user_panel(), 0, 0, 3, 1)
-        grid_layout.addWidget(self._create_metadata_group_box(), 0, 1, 1, 1)
-        grid_layout.addWidget(self._create_output_group_box(), 1, 1, 1, 1)
-        # Add spacer
-        grid_layout.addWidget(QtWidgets.QLabel(), 2, 1, 1, 1)
-        return grid_layout
+        layout_right = QtWidgets.QVBoxLayout()
+        layout_right.addWidget(self._create_metadata_group_box())
+        layout_right.addWidget(self._create_output_group_box())
+        layout = QtWidgets.QHBoxLayout()
+        layout.addWidget(self.create_user_panel())
+        layout.addLayout(layout_right)
+        return layout
 
     def _create_output_group_box(self):
-        group_box = QtWidgets.QGroupBox("Output", self)
-        group_box.setLayout(QtWidgets.QGridLayout(group_box))
+        group_box = QtWidgets.QGroupBox("Output")
         # Create fields
-        output_suffix = QtWidgets.QLineEdit(group_box)
+        output_suffix = LineEdit()
         output_suffix.setStatusTip(
             "(optional) File name suffix in the " "format datetime_suffix.tar"
         )
@@ -183,19 +188,19 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         for w in (QtWidgets.QLabel(LABEL_COMPRESSION_LEVEL),) + slider_widgets:
             compression_layout.addWidget(w)
 
-        group_box.layout().addWidget(QtWidgets.QLabel("Suffix"), 0, 0, 1, 1)
-        group_box.layout().addWidget(output_suffix, 0, 1, 1, 1)
-        group_box.layout().addWidget(QtWidgets.QLabel("Location"), 1, 0, 1, 1)
-        group_box.layout().addWidget(output_location.text, 1, 1, 1, 1)
-        group_box.layout().addWidget(output_location.btn, 2, 1, 1, 1)
-        group_box.layout().addLayout(compression_layout, 3, 0, 1, 2)
+        layout = QtWidgets.QGridLayout(group_box)
+        layout.addWidget(QtWidgets.QLabel("Suffix"), 0, 0, 1, 1)
+        layout.addWidget(output_suffix, 0, 1, 1, 1)
+        layout.addWidget(QtWidgets.QLabel("Location"), 1, 0, 1, 1)
+        layout.addWidget(output_location.text, 1, 1, 1, 1)
+        layout.addWidget(output_location.btn, 1, 2, 1, 1)
+        layout.addLayout(compression_layout, 2, 0, 1, 2)
         return group_box
 
     def _create_metadata_group_box(self):
-        group_box = QtWidgets.QGroupBox("Metadata", self)
-        group_box.setLayout(QtWidgets.QGridLayout(group_box))
+        group_box = QtWidgets.QGroupBox("Metadata")
         # Create fields
-        transfer_id = QtWidgets.QLineEdit(group_box)
+        transfer_id = LineEdit()
         transfer_id.setStatusTip("(optional) Data Transfer Request ID")
         transfer_id.setValidator(QtGui.QIntValidator(1, 10 ** 9))
         verify_dtr = create_verify_dtr_checkbox(
@@ -216,11 +221,12 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
                 self.app_data, "encrypt_purpose", None if text == "" else Purpose(text)
             )
         )
-        group_box.layout().addWidget(QtWidgets.QLabel("DTR ID"), 0, 0, 1, 1)
-        group_box.layout().addWidget(transfer_id, 0, 1, 1, 1)
-        group_box.layout().addWidget(verify_dtr, 1, 0, 1, 2)
-        group_box.layout().addWidget(QtWidgets.QLabel("Purpose"), 2, 0, 1, 1)
-        group_box.layout().addWidget(purpose, 2, 1, 1, 1)
+        layout = QtWidgets.QGridLayout(group_box)
+        layout.addWidget(QtWidgets.QLabel("DTR ID"), 0, 0, 1, 1)
+        layout.addWidget(transfer_id, 0, 1, 1, 1)
+        layout.addWidget(verify_dtr, 1, 0, 1, 2)
+        layout.addWidget(QtWidgets.QLabel("Purpose"), 2, 0, 1, 1)
+        layout.addWidget(purpose, 2, 1, 1, 1)
         return group_box
 
     def encrypt(self, dry_run=False):
@@ -247,7 +253,7 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         worker = Worker(
             encrypt.encrypt,
             self.app_data.encrypt_files,
-            logger=(encrypt.logger,),
+            capture_loggers=(encrypt.logger,),
             sender=self.app_data.encrypt_sender,
             recipient=self.app_data.encrypt_recipients,
             dtr_id=self.app_data.encrypt_transfer_id,
@@ -260,7 +266,8 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
             compression_level=self.app_data.encrypt_compression_level,
             purpose=self.app_data.encrypt_purpose,
             progress=progress,
-            on_error=lambda _: None,
+            ignore_exceptions=True,
+            report_config=self.app_data.config,
         )
         self.add_worker_actions(worker)
         self.threadpool.start(worker)
