@@ -1,11 +1,11 @@
 import html
 from pathlib import Path
-from typing import Optional, Sequence
+from typing import Iterator, Optional, Sequence, Callable
 
 from sett.core.error import UserError
-from .component import SelectionButton
+from .component import SelectionAction, ToolBar, LineEdit, Action, warning_callback
 from .parallel import Worker
-from .pyside import QtCore, QtGui, QtWidgets
+from .pyside import QtCore, QtGui, QtWidgets, QAction
 from ..core import crypt
 from ..utils.config import Config
 from ..workflows import (
@@ -16,10 +16,6 @@ from ..workflows import (
 Key = crypt.gpg.Key
 
 
-def raise_(e: Exception):
-    raise e
-
-
 class KeysTab(QtWidgets.QWidget):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -52,13 +48,19 @@ class KeysTab(QtWidgets.QWidget):
             lambda: self.priv_keys_view.selectionModel().clear()
         )
 
-        btn_generate = QtWidgets.QPushButton("Generate new private/public key")
-        btn_generate.clicked.connect(lambda: KeyGenDialog(parent=self).show())
-        btn_group_public = self.create_public_keys_btn_group()
-        btn_refresh = QtWidgets.QPushButton("Refresh keys")
-        btn_refresh.setStatusTip("Refresh keys from the local keyring")
-        btn_refresh.clicked.connect(
-            lambda x: (
+        action_generate_key = Action(
+            QtGui.QIcon(":icon/feather/plus-square.png"),
+            "Generate new private/public key",
+            self,
+        )
+        action_generate_key.triggered.connect(lambda: KeyGenDialog(parent=self).show())
+        action_refresh_keys = QAction(
+            QtGui.QIcon(":icon/feather/refresh-cw.png"),
+            "Refresh keys from the local keyring",
+            self,
+        )
+        action_refresh_keys.triggered.connect(
+            lambda: (
                 self.update_private_keys(),
                 self.update_public_keys(),
                 self._update_display(
@@ -67,17 +69,21 @@ class KeysTab(QtWidgets.QWidget):
             )
         )
 
+        toolbar = ToolBar("Key management", self)
+        toolbar.addAction(action_generate_key)
+        toolbar.addSeparator()
+        for action in self.create_public_keys_actions():
+            toolbar.addAction(action)
+        toolbar.addSeparator()
+        toolbar.addAction(action_refresh_keys)
+
         layout = QtWidgets.QGridLayout()
-        layout.addWidget(QtWidgets.QLabel("Private keys"), 0, 0)
-        layout.addWidget(self.priv_keys_view, 1, 0)
-        layout.addWidget(btn_generate, 1, 1)
-        layout.addWidget(QtWidgets.QLabel("Public keys"), 2, 0)
-        layout.addWidget(self.pub_keys_view, 3, 0)
-        layout.addLayout(btn_group_public, 3, 1)
-        layout.addWidget(self.text_panel, 4, 0)
-        layout.addWidget(btn_refresh, 4, 1)
-        layout.setRowStretch(3, 1)
-        layout.setRowStretch(4, 1)
+        layout.addWidget(toolbar, 0, 0, 1, 2)
+        layout.addWidget(QtWidgets.QLabel("Private keys"), 1, 0)
+        layout.addWidget(self.priv_keys_view, 2, 0)
+        layout.addWidget(QtWidgets.QLabel("Public keys"), 1, 1)
+        layout.addWidget(self.pub_keys_view, 2, 1)
+        layout.addWidget(self.text_panel, 3, 0, 1, 2)
         self.setLayout(layout)
 
     def key_to_html(self, key: Key) -> str:
@@ -124,52 +130,72 @@ class KeysTab(QtWidgets.QWidget):
     def key_to_text(key: Key) -> str:
         return f"User ID: {key.uids[0]}\n" f"Fingerprint: {key.fingerprint}"
 
-    def create_public_keys_btn_group(self):
+    def create_public_keys_actions(self) -> Iterator[QAction]:
         selection_model = self.pub_keys_view.selectionModel()
 
-        def offline_btn(label: str, tip: str, selection: bool = True):
+        def offline_action(icon: str, tip: str, selection: bool = True):
             """Force disable button in offline mode"""
+            icon_obj = f":icon/feather/{icon}.png"
             if self.app_data.config.offline:
-                btn = QtWidgets.QPushButton(label)
-                btn.setEnabled(False)
-                btn.setStatusTip(f"{tip} (not available in the offline mode)")
+                action = Action(
+                    QtGui.QIcon(icon_obj),
+                    f"{tip} (not available in the offline mode)",
+                    self,
+                )
+                action.setEnabled(False)
             else:
                 if selection:
-                    btn = SelectionButton(label, selection_model)
+                    action = SelectionAction(
+                        QtGui.QIcon(icon_obj),
+                        tip,
+                        self,
+                        selection_model=selection_model,
+                    )
                 else:
-                    btn = QtWidgets.QPushButton(label)
-                btn.setStatusTip(tip)
-            return btn
+                    action = Action(QtGui.QIcon(icon_obj), tip, self)
+            return action
 
-        btn_download = offline_btn(
-            "Download keys", "Search and download keys from the keyserver", False
-        )
-        btn_download.clicked.connect(lambda: KeyDownloadDialog(parent=self).show())
-        btn_update = offline_btn(
-            "Update keys", "Update selected keys from the keyserver"
-        )
-        btn_update.clicked.connect(self.update_keys)
-        btn_import = QtWidgets.QPushButton("Import key")
-        btn_import.setStatusTip("Import key from file")
-        btn_import.clicked.connect(self.import_key)
-        btn_upload = offline_btn("Upload keys", "Upload selected keys to the keyserver")
-        btn_upload.clicked.connect(self.upload_key)
-        btn_sign = offline_btn(
-            "Request signature", "Request signature for the selected keys"
-        )
-        btn_sign.clicked.connect(self.send_signature_request)
-        btn_delete = SelectionButton("Delete keys", selection_model)
-        btn_delete.setStatusTip("Delete selected keys from your computer")
-        btn_delete.clicked.connect(self.delete_keys)
-
-        btn_group_public = QtWidgets.QVBoxLayout()
-        btn_group_public.addWidget(btn_download)
-        btn_group_public.addWidget(btn_update)
-        btn_group_public.addWidget(btn_import)
-        btn_group_public.addWidget(btn_upload)
-        btn_group_public.addWidget(btn_sign)
-        btn_group_public.addWidget(btn_delete)
-        return btn_group_public
+        for action, fn in (
+            (
+                offline_action(
+                    "download-cloud",
+                    "Search and download keys from the keyserver",
+                    False,
+                ),
+                lambda: KeyDownloadDialog(parent=self).show(),
+            ),
+            (
+                offline_action("upload-cloud", "Upload selected keys to the keyserver"),
+                self.upload_key,
+            ),
+            (
+                offline_action("pen-tool", "Request signature for the selected keys"),
+                self.send_signature_request,
+            ),
+            (
+                offline_action("rotate-cw", "Update selected keys from the keyserver"),
+                self.update_keys,
+            ),
+            (
+                SelectionAction(
+                    QtGui.QIcon(":icon/feather/trash-2.png"),
+                    "Delete selected keys from your computer",
+                    self,
+                    selection_model=selection_model,
+                ),
+                self.delete_keys,
+            ),
+            (
+                Action(
+                    QtGui.QIcon(":icon/feather/file-plus.png"),
+                    "Import key from file",
+                    self,
+                ),
+                self.import_key,
+            ),
+        ):
+            action.triggered.connect(fn)
+            yield action
 
     def update_private_keys(self):
         # Retrieve all private keys from local keyring.
@@ -200,13 +226,8 @@ class KeysTab(QtWidgets.QWidget):
 
     def update_keys(self):
         """Updated selected keys from the keyserver."""
-        msg_ok = QtWidgets.QMessageBox()
-        msg_ok.setIcon(QtWidgets.QMessageBox.Information)
-        msg_ok.setWindowTitle("Updated keys")
-        msg_ok.setText("Keys have been successfully updated.")
-        msg_warn = QtWidgets.QMessageBox()
-        msg_warn.setWindowTitle("GPG key update error")
-        msg_warn.setIcon(QtWidgets.QMessageBox.Warning)
+        show_ok = ok_message("Updated keys", "Keys have been successfully updated.")
+        show_warning = warning_callback("GPG key update error")
 
         selected_keys = [
             index.model().get_value(index).fingerprint
@@ -218,6 +239,8 @@ class KeysTab(QtWidgets.QWidget):
                 selected_keys,
                 self.app_data.config.keyserver_url,
                 self.app_data.config.gpg_store,
+                report_config=self.app_data.config,
+                forward_errors=show_warning,
             )
             worker.signals.result.connect(
                 lambda x: (
@@ -225,12 +248,9 @@ class KeysTab(QtWidgets.QWidget):
                     self._update_display(
                         self.pub_keys_view.selectionModel().currentIndex()
                     ),
-                    msg_ok.exec_(),
+                    show_ok(),
                 )
             )
-            worker.signals.error.connect(
-                lambda x: (msg_warn.setText(format(x[1])), msg_warn.exec_())
-            )
             self.threadpool.start(worker)
 
     def import_key(self):
@@ -301,13 +321,10 @@ class KeysTab(QtWidgets.QWidget):
         msg.setText("Do you want to upload the selected key to the key server?")
         msg.setIcon(QtWidgets.QMessageBox.Question)
         msg.setStandardButtons(QtWidgets.QMessageBox.Ok | QtWidgets.QMessageBox.Cancel)
-        msg_ok = QtWidgets.QMessageBox()
-        msg_ok.setIcon(QtWidgets.QMessageBox.Information)
-        msg_ok.setWindowTitle("Send public key")
-        msg_ok.setText("Key has been successfully uploaded to the keyserver.")
-        msg_warn = QtWidgets.QMessageBox()
-        msg_warn.setWindowTitle("GPG key upload error")
-        msg_warn.setIcon(QtWidgets.QMessageBox.Warning)
+        show_ok = ok_message(
+            "Send public key", "Key has been successfully uploaded to the keyserver."
+        )
+        show_warning = warning_callback("GPG key upload error")
 
         selected_keys = [
             index.model().get_value(index)
@@ -323,13 +340,11 @@ class KeysTab(QtWidgets.QWidget):
                     upload_keys_workflow.upload_keys,
                     [key.fingerprint],
                     config=self.app_data.config,
-                    logger=(upload_keys_workflow.logger,),
-                    on_error=raise_,
-                )
-                worker.signals.result.connect(lambda x: msg_ok.exec_())
-                worker.signals.error.connect(
-                    lambda x: (msg_warn.setText(format(x[1])), msg_warn.exec_())
+                    report_config=self.app_data.config,
+                    capture_loggers=(upload_keys_workflow.logger,),
+                    forward_errors=show_warning,
                 )
+                worker.signals.result.connect(lambda _: show_ok())
                 self.threadpool.start(worker)
 
     def send_signature_request(self):
@@ -363,7 +378,7 @@ class KeyDownloadDialog(QtWidgets.QDialog):
         self.setWindowTitle("Download public keys from the server")
         self.setWindowFlags(self.windowFlags() & ~QtCore.Qt.WindowContextHelpButtonHint)
 
-        self.search_string = QtWidgets.QLineEdit()
+        self.search_string = LineEdit()
         self.btn_search = QtWidgets.QPushButton("Search")
         self.btn_search.clicked.connect(self.search_keys)
 
@@ -400,6 +415,7 @@ class KeyDownloadDialog(QtWidgets.QDialog):
             crypt.search_keyserver,
             self.search_string.text(),
             self.parent().app_data.config.keyserver_url,
+            report_config=self.app_data.config,
         )
         worker.signals.result.connect(
             lambda keys: self.key_list_view.model().setStringList(
@@ -420,6 +436,7 @@ class KeyDownloadDialog(QtWidgets.QDialog):
                 key_ids,
                 self.parent().app_data.config.keyserver_url,
                 self.parent().app_data.config.gpg_store,
+                report_config=self.app_data.config,
             )
             worker.signals.result.connect(lambda x: self.parent().update_public_keys())
             worker.signals.finished.connect(lambda: self.btn_download.setEnabled(True))
@@ -433,11 +450,11 @@ class KeyGenDialog(QtWidgets.QDialog):
         self.setWindowTitle("Generate new key pair")
         self.setWindowFlags(self.windowFlags() & ~QtCore.Qt.WindowContextHelpButtonHint)
 
-        self.text_name_full = QtWidgets.QLineEdit()
-        self.text_name_extra = QtWidgets.QLineEdit()
-        self.text_email = QtWidgets.QLineEdit()
-        self.text_pass = QtWidgets.QLineEdit()
-        self.text_pass_repeat = QtWidgets.QLineEdit()
+        self.text_name_full = LineEdit()
+        self.text_name_extra = LineEdit()
+        self.text_email = LineEdit()
+        self.text_pass = LineEdit()
+        self.text_pass_repeat = LineEdit()
         self.toggle_password_visibility(False)
 
         re_email = QtCore.QRegularExpression(r"[^@]+@[^@]+\.[^@]+")
@@ -530,9 +547,7 @@ class KeyGenDialog(QtWidgets.QDialog):
             self.close()
 
     def create_private_key(self):
-        msg_warn = QtWidgets.QMessageBox()
-        msg_warn.setWindowTitle("GPG Key Generation Error")
-        msg_warn.setIcon(QtWidgets.QMessageBox.Warning)
+        show_warning = warning_callback("GPG Key Generation Error")
         self.btn_run.setEnabled(False)
         full_name = f"{self.text_name_full.text()} " f"{self.text_name_extra.text()}"
         worker = Worker(
@@ -542,11 +557,10 @@ class KeyGenDialog(QtWidgets.QDialog):
             self.text_pass.text(),
             self.text_pass_repeat.text(),
             self.parent().app_data.config.gpg_store,
+            forward_errors=show_warning,
+            report_config=self.app_data.config,
         )
         worker.signals.result.connect(self.post_key_creation)
-        worker.signals.error.connect(
-            lambda e: (msg_warn.setText(format(e[1])), msg_warn.exec_())
-        )
         worker.signals.finished.connect(lambda: self.btn_run.setEnabled(True))
         self.threadpool.start(worker)
 
@@ -568,17 +582,10 @@ def request_signature(
     msg.setText("Do you want to request signature for this key?")
     msg.setIcon(QtWidgets.QMessageBox.Question)
     msg.setStandardButtons(QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
-    msg_ok = QtWidgets.QMessageBox(parent)
-    msg_ok.setIcon(QtWidgets.QMessageBox.Information)
-    msg_ok.setWindowTitle("Key signing request")
-    msg_ok.setText("Key signing request has been sent.")
-    msg_warn = QtWidgets.QMessageBox(parent)
-    msg_warn.setWindowTitle("Key signing request")
-    msg_warn.setIcon(QtWidgets.QMessageBox.Warning)
-
-    def show_warning(e):
-        msg_warn.setText(format(e[1]))
-        msg_warn.exec_()
+    show_ok = ok_message(
+        "Key signing request", "Key signing request has been sent.", parent=parent
+    )
+    show_warning = warning_callback("Key signing request", parent=parent)
 
     for key in keys:
         msg.setDetailedText(KeysTab.key_to_text(key))
@@ -590,9 +597,17 @@ def request_signature(
                 request_sigs_workflow.request_sigs,
                 key_ids=[key.key_id],
                 config=config,
-                logger=(request_sigs_workflow.logger,),
-                on_error=raise_,
+                capture_loggers=(request_sigs_workflow.logger,),
+                forward_errors=show_warning,
+                report_config=config,
             )
-            worker.signals.result.connect(lambda _: msg_ok.exec_())
-            worker.signals.error.connect(show_warning)
+            worker.signals.result.connect(lambda _: show_ok())
             threadpool.start(worker)
+
+
+def ok_message(title: str, msg: str, parent=None) -> Callable:
+    msg_ok = QtWidgets.QMessageBox(parent)
+    msg_ok.setIcon(QtWidgets.QMessageBox.Information)
+    msg_ok.setWindowTitle(title)
+    msg_ok.setText(msg)
+    return msg_ok.exec_
