@@ -9,16 +9,16 @@ from sett import (
     URL_GITLAB,
     VERSION_WITH_DEPS,
 )
-from .component import show_warning
-from .settings_tab import SettingsTab, config_to_settings
-from ..core.versioncheck import check_version
-from .keys_tab import KeysTab
-from .transfer_tab import TransferTab
+from .component import show_warning, is_macos
 from .decrypt_tab import DecryptTab
 from .encrypt_tab import EncryptTab
+from .keys_tab import KeysTab
 from .model import AppData
 from .parallel import Worker
 from .pyside import QtCore, QtGui, QtWidgets, QAction
+from .settings_tab import SettingsTab, config_to_settings
+from .transfer_tab import TransferTab
+from ..core.versioncheck import check_version
 from ..utils.config import load_config, save_config, config_to_dict
 
 QtCore.QThreadPool.globalInstance().setExpiryTimeout(-1)
@@ -30,6 +30,7 @@ class MainWindow(QtWidgets.QMainWindow):
         self.title = f"{APP_NAME_LONG} ({__version__})"
         self.app_data = AppData(config=self.get_config())
         self.setWindowTitle(self.title)
+        self.setMinimumSize(QtCore.QSize(900, 900))
         self.add_menu()
         self.add_tabs()
         self.add_status_bar()
@@ -54,11 +55,13 @@ class MainWindow(QtWidgets.QMainWindow):
         tab_settings = SettingsTab(self)
 
         tabs = QtWidgets.QTabWidget()
-        tabs.addTab(tab_encrypt, "&Encrypt")
-        tabs.addTab(tab_transfer, "&Transfer")
-        tabs.addTab(tab_decrypt, "&Decrypt")
-        tabs.addTab(tab_keys, "&Keys")
-        tabs.addTab(tab_settings, "&Settings")
+        tabs.addTab(tab_encrypt, QtGui.QIcon(":icon/feather/lock.png"), "&Encrypt")
+        tabs.addTab(tab_transfer, QtGui.QIcon(":icon/feather/send.png"), "&Transfer")
+        tabs.addTab(tab_decrypt, QtGui.QIcon(":icon/feather/unlock.png"), "&Decrypt")
+        tabs.addTab(tab_keys, QtGui.QIcon(":icon/feather/key.png"), "&Keys")
+        tabs.addTab(
+            tab_settings, QtGui.QIcon(":icon/feather/settings.png"), "&Settings"
+        )
 
         self.setCentralWidget(tabs)
 
@@ -67,26 +70,30 @@ class MainWindow(QtWidgets.QMainWindow):
         self.setStatusBar(self.status)
 
     def add_menu(self):
-        action_exit = QAction(QtGui.QIcon(), "&Exit", self)
+        action_exit = QAction(QtGui.QIcon(":icon/feather/log-out.png"), "&Exit", self)
         action_exit.setShortcut(QtGui.QKeySequence("Ctrl+Q"))
         action_exit.setStatusTip("Exit application")
         action_exit.triggered.connect(self.close)
 
         menu = self.menuBar()
-        menu.setNativeMenuBar(QtCore.QSysInfo.productType() != "macos")
+        menu.setNativeMenuBar(not is_macos())
         menu_file = menu.addMenu("&File")
         menu_file.addAction(action_exit)
 
-        action_help = QAction(QtGui.QIcon(), "&Documentation", self)
+        action_help = QAction(
+            QtGui.QIcon(":icon/feather/book-open.png"), "&Documentation", self
+        )
         action_help.setStatusTip("Open online documentation")
         action_help.setShortcut(QtGui.QKeySequence(QtGui.QKeySequence.HelpContents))
         action_help.triggered.connect(open_url(URL_READTHEDOCS))
 
-        action_bug_report = QAction(QtGui.QIcon(), "&Report an Issue", self)
+        action_bug_report = QAction(
+            QtGui.QIcon(":icon/feather/alert-triangle.png"), "&Report an Issue", self
+        )
         action_bug_report.setStatusTip("Open online bug report form")
         action_bug_report.triggered.connect(open_url(URL_GITLAB_ISSUES))
 
-        action_about = QAction(QtGui.QIcon(), "&About", self)
+        action_about = QAction(QtGui.QIcon(":icon/feather/info.png"), "&About", self)
         action_about.setStatusTip("Show info about application")
         action_about.triggered.connect(self.show_about)
 
@@ -132,7 +139,10 @@ class MainWindow(QtWidgets.QMainWindow):
                 check_version(self.app_data.config.repo_url, gui_formatting=True)
                 return "\n".join(format(warning.message) for warning in w)
 
-        worker = Worker(get_warnings)
+        worker = Worker(
+            get_warnings,
+            report_config=self.app_data.config,
+        )
         worker.signals.result.connect(lambda x: x and show_warning(self.title, x, self))
         QtCore.QThreadPool.globalInstance().start(worker)
 
