@@ -1,6 +1,6 @@
 from typing import Optional, Dict
 
-from .component import PathInput, FieldMapping, create_slider
+from .component import PathInput, FieldMapping, create_slider, LineEdit, SpinBox
 from .pyside import QtCore, QtGui, QtWidgets
 from ..utils import config
 from ..utils.config import Config
@@ -218,8 +218,16 @@ class SettingsTab(QtWidgets.QWidget):
             1,
         )
 
+        self._add_widget_bool(
+            misc_log_group,
+            "Create error reports",
+            "error_reports",
+            "Write an error report if some error happens",
+        )
+
     def _add_footer(self):
         persist_btn = QtWidgets.QPushButton(self.persist_btn_text, self)
+        persist_btn.setIcon(QtGui.QIcon(":icon/feather/save.png"))
         persist_btn.setStatusTip("Saves your current settings to the config file")
         persist_btn.clicked.connect(
             lambda: config.save_config(config.config_to_dict(self.config))
@@ -248,7 +256,7 @@ class SettingsTab(QtWidgets.QWidget):
         status_tip: str,
         regex: Optional[str] = None,
     ):
-        widget = self.field_mapping.bind_text(config_key, QtWidgets.QLineEdit())
+        widget = self.field_mapping.bind_text(config_key, LineEdit())
         widget.setStatusTip(status_tip)
         if regex:
             widget.setValidator(
@@ -276,7 +284,6 @@ class SettingsTab(QtWidgets.QWidget):
         widget = self.field_mapping.bind_path(
             config_key, PathInput(directory=directory, path=None)
         )
-        widget.btn_label = "Select directory" if directory else "Select file"
         widget.status_tip = status_tip
 
         widget.on_path_change(
@@ -320,7 +327,7 @@ class SettingsTab(QtWidgets.QWidget):
         minimum: Optional[int] = None,
         maximum: Optional[int] = None,
     ):
-        widget = self.field_mapping.bind_value(config_key, QtWidgets.QSpinBox())
+        widget = self.field_mapping.bind_value(config_key, SpinBox())
         widget.setStatusTip(status_tip)
         if minimum is not None:
             widget.setMinimum(minimum)
