@@ -1,9 +1,8 @@
 import enum
 import operator
-from functools import partial
-from typing import Optional, Sequence
-
 import time
+from functools import partial
+from typing import Iterator, Optional, Sequence
 
 from .component import (
     TabMixin,
@@ -13,10 +12,13 @@ from .component import (
     MandatoryLabel,
     create_verify_dtr_checkbox,
     FieldMapping,
+    ToolBar,
+    LineEdit,
+    Action,
 )
 from .file_selection_widget import ArchiveOnlyFileSelectionWidget
 from .parallel import Worker
-from .pyside import QtCore, QtGui, QtWidgets
+from .pyside import QtCore, QtGui, QtWidgets, QAction
 from .. import protocols
 from ..core.error import UserError
 from ..utils.config import Connection, ConnectionStore
@@ -32,21 +34,29 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
         self.connections_model = QtCore.QStringListModel()
 
         files_panel = self.create_files_panel()
-        verify_dtr = create_verify_dtr_checkbox(self.app_data, "transfer_verify_dtr")
+
+        verify_dtr_group = QtWidgets.QGroupBox()
+        verify_dtr_layout = QtWidgets.QVBoxLayout(verify_dtr_group)
+        verify_dtr_layout.addWidget(
+            create_verify_dtr_checkbox(self.app_data, "transfer_verify_dtr")
+        )
+
         options_panel = self.create_options_panel()
         self.create_run_panel("Transfer data", self.transfer, "Transfer selected files")
         self.app_data.add_listener("transfer_files", self._enable_buttons)
         self.create_console()
         self.create_progress_bar()
 
-        layout = QtWidgets.QGridLayout()
-        layout.addWidget(files_panel, 0, 0, 1, 2)
-        layout.addWidget(verify_dtr, 1, 0)
-        layout.addWidget(options_panel, 2, 0, 1, 2)
-        layout.addWidget(self.run_panel, 3, 0, 1, 2)
-        layout.addWidget(self.console, 4, 0, 1, 2)
-        layout.addWidget(self.progress_bar, 5, 0, 1, 2)
-        self.setLayout(layout)
+        layout = QtWidgets.QVBoxLayout(self)
+        for widget in (
+            files_panel,
+            verify_dtr_group,
+            options_panel,
+            self.run_panel,
+            self.console,
+            self.progress_bar,
+        ):
+            layout.addWidget(widget)
 
     def _enable_buttons(self):
         self.set_buttons_enabled(len(self.app_data.transfer_files) > 0)
@@ -63,11 +73,9 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
     def create_sftp_options_panel(self):
         field_mapping = FieldMapping()
 
-        text_username = field_mapping.bind_text("username", QtWidgets.QLineEdit())
+        text_username = field_mapping.bind_text("username", LineEdit())
         text_username.setStatusTip("Username on the SFTP server")
-        text_destination_dir = field_mapping.bind_text(
-            "destination_dir", QtWidgets.QLineEdit()
-        )
+        text_destination_dir = field_mapping.bind_text("destination_dir", LineEdit())
         text_username.textChanged.connect(
             lambda: operator.setitem(
                 self.app_data.transfer_protocol_args["sftp"],
@@ -75,14 +83,14 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
                 text_username.text(),
             )
         )
-        text_host = field_mapping.bind_text("host", QtWidgets.QLineEdit())
+        text_host = field_mapping.bind_text("host", LineEdit())
         text_host.setStatusTip("URL of the SFTP server with an optional port " "number")
         text_host.textChanged.connect(
             lambda: operator.setitem(
                 self.app_data.transfer_protocol_args["sftp"], "host", text_host.text()
             )
         )
-        text_jumphost = field_mapping.bind_text("jumphost", QtWidgets.QLineEdit())
+        text_jumphost = field_mapping.bind_text("jumphost", LineEdit())
         text_jumphost.setStatusTip("(optional) URL of the jumphost server")
         text_jumphost.textChanged.connect(
             lambda: operator.setitem(
@@ -106,7 +114,6 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
         pkey_location = field_mapping.bind_path(
             "pkey", PathInput(directory=False, path=None)
         )
-        pkey_location.btn_label = "Select private key"
         pkey_location.status_tip = "Path to the private SSH key used for authentication"
         pkey_location.on_path_change(
             lambda path: operator.setitem(
@@ -115,7 +122,7 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
                 "" if path is None else str(path),
             )
         )
-        pkey_password = field_mapping.bind_text("pkey_password", QtWidgets.QLineEdit())
+        pkey_password = field_mapping.bind_text("pkey_password", LineEdit())
         pkey_password.setStatusTip("Passphrase for the SSH private key")
         pkey_password.setEchoMode(QtWidgets.QLineEdit.Password)
         pkey_password.textChanged.connect(
@@ -126,7 +133,10 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
             )
         )
 
-        layout = QtWidgets.QGridLayout()
+        box = QtWidgets.QGroupBox()
+        box.setFlat(True)
+        box.load_connection = field_mapping.load
+        layout = QtWidgets.QGridLayout(box)
         layout.addWidget(MandatoryLabel("User name"), 0, 0)
         layout.addWidget(text_username, 0, 1)
         layout.addWidget(MandatoryLabel("Host URL"), 1, 0)
@@ -141,16 +151,12 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
         layout.addWidget(pkey_location.btn_clear, 4, 3)
         layout.addWidget(QtWidgets.QLabel("SSH key password"), 5, 0)
         layout.addWidget(pkey_password, 5, 1)
-        box = QtWidgets.QGroupBox("Parameters")
-        box.setFlat(True)
-        box.setLayout(layout)
-        box.load_connection = field_mapping.load
         return box
 
     def create_liquidfiles_options_panel(self):
         field_mapping = FieldMapping()
-        text_host = field_mapping.bind_text("host", QtWidgets.QLineEdit())
-        text_api_key = field_mapping.bind_text("api_key", QtWidgets.QLineEdit())
+        text_host = field_mapping.bind_text("host", LineEdit())
+        text_api_key = field_mapping.bind_text("api_key", LineEdit())
         text_host.textChanged.connect(
             lambda: operator.setitem(
                 self.app_data.transfer_protocol_args["liquid_files"],
@@ -165,18 +171,19 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
                 text_api_key.text(),
             )
         )
-        layout = QtWidgets.QGridLayout()
+        box = QtWidgets.QGroupBox()
+        box.setFlat(True)
+        box.load_connection = field_mapping.load
+        layout = QtWidgets.QGridLayout(box)
         layout.addWidget(MandatoryLabel("Host URL"), 0, 0)
         layout.addWidget(text_host, 0, 1)
         layout.addWidget(MandatoryLabel("API Key"), 1, 0)
         layout.addWidget(text_api_key, 1, 1)
-        box = QtWidgets.QGroupBox("Parameters")
-        box.setFlat(True)
-        box.setLayout(layout)
-        box.load_connection = field_mapping.load
         return box
 
-    def create_conn_btn_grp(self, connections_selection: QtWidgets.QComboBox):
+    def create_conn_actions(
+        self, connections_selection: QtWidgets.QComboBox
+    ) -> Iterator[QAction]:
         connection_store = ConnectionStore()
 
         def confirm(text: str) -> bool:
@@ -258,26 +265,21 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
         def set_btn_enabled(btn: QtWidgets.QAbstractButton, text: str):
             btn.setEnabled(bool(text))
 
-        buttons = (
-            ("New", "Create a new connection profile", add, False),
-            ("Rename", "Rename the current connection profile", rename, True),
-            ("Delete", "Delete the current connection profile", delete, True),
-            ("Save", "Save the current connection profile", save, True),
-        )
-        grp = QtWidgets.QButtonGroup()
-        for label, tip, action, needs_selection in buttons:
-            btn = QtWidgets.QPushButton(label)
-            btn.setStatusTip(tip)
-            btn.clicked.connect(action)
+        for tip, fn, needs_selection, icon in (
+            ("Create a new connection profile", add, False, "plus-square"),
+            ("Rename the current connection profile", rename, True, "edit"),
+            ("Delete the current connection profile", delete, True, "trash-2"),
+            ("Save the current connection profile", save, True, "save"),
+        ):
+            action = Action(QtGui.QIcon(f":icon/feather/{icon}.png"), tip, self)
+            action.triggered.connect(fn)
             if needs_selection:
                 connections_selection.currentTextChanged.connect(
-                    partial(set_btn_enabled, btn)
+                    partial(set_btn_enabled, action)
                 )
                 if not connections_selection.currentText():
-                    btn.setEnabled(False)
-            grp.addButton(btn)
-
-        return grp
+                    action.setEnabled(False)
+            yield action
 
     def create_options_panel(self):
         box = QtWidgets.QGroupBox("Connection")
@@ -321,12 +323,12 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
 
         protocol_btn_grp.buttonToggled.connect(toggle_protocol)
 
-        layout_connection = QtWidgets.QGridLayout()
-        layout_connection.addWidget(connections_selection, 0, 0, 1, 2)
-        for i, btn in enumerate(
-            self.create_conn_btn_grp(connections_selection).buttons(), 3
-        ):
-            layout_connection.addWidget(btn, 0, i)
+        toolbar = ToolBar("Options", self)
+        for action in self.create_conn_actions(connections_selection):
+            toolbar.addAction(action)
+        layout_connection = QtWidgets.QHBoxLayout()
+        layout_connection.addWidget(connections_selection)
+        layout_connection.addWidget(toolbar)
 
         layout_protocol_buttons = QtWidgets.QHBoxLayout()
         for btn in protocol_btn_grp.buttons():
@@ -336,13 +338,10 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
         layout_protocol.addLayout(layout_protocol_buttons)
         for protocol_box in protocol_boxes.values():
             layout_protocol.addWidget(protocol_box)
-        box_protocol = QtWidgets.QGroupBox("Protocol")
-        box_protocol.setLayout(layout_protocol)
 
-        layout = QtWidgets.QVBoxLayout()
+        layout = QtWidgets.QVBoxLayout(box)
         layout.addLayout(layout_connection)
-        layout.addWidget(box_protocol)
-        box.setLayout(layout)
+        layout.addLayout(layout_protocol)
         load_connection()
         return box
 
@@ -377,22 +376,30 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
                 output = run_dialog(self, "Verification code", password=False)
                 second_factor = "" if output is None else output
 
+        specific_protocol_args = {
+            protocols.sftp.Protocol: {"two_factor_callback": second_factor_callback}
+        }
+        protocol_type = protocols.protocols_by_name[
+            self.app_data.transfer_protocol_name
+        ]
+        protocol = protocol_type(
+            **self.app_data.transfer_protocol_args[
+                self.app_data.transfer_protocol_name
+            ],
+            **specific_protocol_args.get(protocol_type, {}),
+        )
+
         worker = Worker(
             transfer.transfer,
             files=self.app_data.transfer_files,
-            logger=(transfer.logger, protocols.sftp.logger),
-            protocol=protocols.protocols_by_name[
-                self.app_data.transfer_protocol_name
-            ].upload,
-            protocol_args=self.app_data.transfer_protocol_args[
-                self.app_data.transfer_protocol_name
-            ],
+            capture_loggers=(transfer.logger, protocols.sftp.logger),
+            protocol=protocol,
             config=self.app_data.config,
             dry_run=dry_run,
             verify_dtr=self.app_data.transfer_verify_dtr,
             progress=progress,
-            on_error=lambda _: None,
-            two_factor_callback=second_factor_callback,
+            ignore_exceptions=True,
+            report_config=self.app_data.config,
         )
         self.add_worker_actions(worker)
         msg_signal.msg.connect(show_second_factor_dialog)
@@ -419,7 +426,7 @@ class ConnectionDialog(QtWidgets.QDialog):
         btn_box.accepted.connect(self.accept)
         btn_box.rejected.connect(self.reject)
 
-        self.text_field = QtWidgets.QLineEdit()
+        self.text_field = LineEdit()
         self.text_field.setValidator(
             QtGui.QRegularExpressionValidator(QtCore.QRegularExpression(r"[\w\-@]+"))
         )
