@@ -2,6 +2,7 @@ import re
 import os
 import io
 from datetime import datetime
+from dataclasses import dataclass
 from typing import Optional, Callable, Tuple, Sequence
 from pathlib import Path, PurePosixPath
 from contextlib import contextmanager
@@ -23,89 +24,95 @@ from ..utils.progress import ProgressInterface
 from ..utils.log import create_logger
 from ..utils.config import get_config_file
 from ..core.error import UserError
+from ..core.secret import Secret
 from .defs import ENVELOPE_DIR_FMT
+from . import protocol
 
 
 logger = create_logger(__name__)
 
 
-def required_password_args(
-    pkey: Optional[str] = None, pkey_password: Optional[str] = None, **_
-) -> Sequence:
-    if pkey is not None and pkey_password is None:
-        return ("pkey_password",)
-    return ()
+@dataclass
+class Protocol(protocol.Protocol):
+    host: str
+    username: str
+    destination_dir: str
+    two_factor_callback: Callable
+    envelope_dir: Optional[str] = None
+    pkey: Optional[str] = None
+    pkey_password: Optional[Secret] = None
+    pkey_password_encoding: str = (
+        "utf_8"  # nosec (False Positive: pkey_password_encoding)
+    )
+    jumphost: Optional[str] = None
 
+    def required_password_args(self) -> Sequence[str]:
+        if self.pkey is not None and self.pkey_password is None:
+            return ("pkey_password",)
+        return ()
 
-def upload(  # nosec (False Positive: pkey_password_encoding)
-    files: Sequence[str],
-    host: str,
-    username: str,
-    destination_dir: str,
-    two_factor_callback: Callable,
-    envelope_dir: Optional[str] = None,
-    pkey: Optional[str] = None,
-    pkey_password: Optional[str] = None,
-    pkey_password_encoding: str = "utf_8",
-    jumphost: Optional[str] = None,
-    progress: Optional[ProgressInterface] = None,
-):
-    if envelope_dir is None:
-        envelope_dir = datetime.now().strftime(ENVELOPE_DIR_FMT)
-    progress_callback = progress and (lambda x, y: progress.update(x / y))
-    remote_dir = PurePosixPath(destination_dir) / envelope_dir
-    try:
-        with sftp_connection(
-            host=host,
-            username=username,
-            pkey=pkey,
-            jumphost=jumphost,
-            pkey_password=pkey_password,
-            pkey_password_encoding=pkey_password_encoding,
-            two_factor_callback=two_factor_callback,
-        ) as sftp:
-            try:
-                sftp.mkdir(str(remote_dir))
-            except FileNotFoundError as e:
-                raise UserError(
-                    "Remote destination directory does not exist: " f"{destination_dir}"
-                ) from e
-            except PermissionError as e:
-                raise UserError(
-                    "You don not have enough permissions to write "
-                    f"to the remote directory: {destination_dir}"
-                ) from e
-            for tar in files:
-                remotepath = str(remote_dir / Path(tar).name)
-                remotepath_part = remotepath + ".part"
-                status = sftp.put(
-                    localpath=os.path.realpath(tar),
-                    remotepath=remotepath_part,
-                    callback=progress_callback,
-                    confirm=True,
-                )
-                remote_size = status.st_size
-                local_size = os.path.getsize(os.path.realpath(tar))
-
-                if local_size != remote_size:
-                    raise UserError(
-                        f"Incomplete file transfer: '{tar}'\n"
-                        f"Remote: {remote_size}\nLocal: {local_size}"
-                    )
-
+    def upload(
+        self,
+        files: Sequence[str],
+        progress: Optional[ProgressInterface] = None,
+    ):
+        if self.envelope_dir is None:
+            envelope_dir = datetime.now().strftime(ENVELOPE_DIR_FMT)
+        progress_callback = progress and (lambda x, y: progress.update(x / y))
+        remote_dir = PurePosixPath(self.destination_dir) / envelope_dir
+        try:
+            with sftp_connection(
+                host=self.host,
+                username=self.username,
+                pkey=self.pkey,
+                jumphost=self.jumphost,
+                pkey_password=self.pkey_password,
+                pkey_password_encoding=self.pkey_password_encoding,
+                two_factor_callback=self.two_factor_callback,
+            ) as sftp:
                 try:
-                    sftp.posix_rename(remotepath_part, remotepath)
-                except IOError as e:
-                    raise UserError(format(e)) from e
-            with io.BytesIO(b"") as fl:
-                sftp.putfo(
-                    fl=fl,
-                    remotepath=str(remote_dir / "done.txt"),
-                    callback=progress_callback,
-                    confirm=True,
-                )
-    except ssh_exception.AuthenticationException as e:
-        raise UserError(format(e)) from e
+                    sftp.mkdir(str(remote_dir))
+                except FileNotFoundError as e:
+                    raise UserError(
+                        "Remote destination directory does not exist: "
+                        f"{self.destination_dir}"
+                    ) from e
+                except PermissionError as e:
+                    raise UserError(
+                        "You don not have enough permissions to write "
+                        f"to the remote directory: {self.destination_dir}"
+                    ) from e
+                for tar in files:
+                    remotepath = str(remote_dir / Path(tar).name)
+                    remotepath_part = remotepath + ".part"
+                    status = sftp.put(
+                        localpath=os.path.realpath(tar),
+                        remotepath=remotepath_part,
+                        callback=progress_callback,
+                        confirm=True,
+                    )
+                    remote_size = status.st_size
+                    local_size = os.path.getsize(os.path.realpath(tar))
+
+                    if local_size != remote_size:
+                        raise UserError(
+                            f"Incomplete file transfer: '{tar}'\n"
+                            f"Remote: {remote_size}\nLocal: {local_size}"
+                        )
+
+                    try:
+                        sftp.posix_rename(remotepath_part, remotepath)
+                    except IOError as e:
+                        raise UserError(format(e)) from e
+                with io.BytesIO(b"") as fl:
+                    sftp.putfo(
+                        fl=fl,
+                        remotepath=str(remote_dir / "done.txt"),
+                        callback=progress_callback,
+                        confirm=True,
+                    )
+        except ssh_exception.AuthenticationException as e:
+            raise UserError(format(e)) from e
 
 
 @contextmanager
@@ -114,7 +121,7 @@ def sftp_connection(  # nosec
     username: str,
     two_factor_callback: Callable,
     pkey: Optional[str] = None,
-    pkey_password: Optional[str] = None,
+    pkey_password: Optional[Secret] = None,
     pkey_password_encoding: str = "utf_8",
     jumphost: Optional[str] = None,
 ):
@@ -160,7 +167,12 @@ def auth(trans, username, key, two_factor_callback: Callable):
 def proxy_socket(host, jumphost, username, **kwargs):
     tunnel = SSHClient()
     tunnel.set_missing_host_key_policy(AutoAddPolicy())
-    tunnel.connect(jumphost, username=username, allow_agent=True, **kwargs)
+    tunnel.connect(
+        jumphost,
+        username=username,
+        allow_agent=True,
+        **kwargs,
+    )
     return tunnel.get_transport().open_channel(
         "direct-tcpip", parse_host(host), parse_host(jumphost)
     )
