@@ -1,9 +1,8 @@
-from typing import List, Tuple, Callable, Dict, Optional
+from typing import List, Tuple, Dict, Optional
 import json
 
-from . import workflow
 from ..utils.config import Config
-from ..utils.log import create_logger
+from ..utils.log import create_logger, log_timing
 from ..utils.progress import ProgressInterface
 from ..core.crypt import (
     get_recipient_email,
@@ -14,19 +13,17 @@ from ..core.crypt import (
 from ..core.archive import check_tar, extract, METADATA_FILE, DATA_FILE_ENCRYPTED
 from ..core.metadata import load_metadata
 from ..core.error import UserError
-from ..protocols import needs_argument
+from ..protocols import Protocol
 
 
 logger = create_logger(__name__)
 
 
-@workflow(logger, (UserError, FileNotFoundError))
+@log_timing(logger)
 def transfer(
     files: List[str],
     *,
-    protocol,
-    two_factor_callback: Callable,
-    protocol_args: dict,
+    protocol: Protocol,
     config: Config,
     dry_run: bool = False,
     verify_dtr: Optional[bool] = None,
@@ -90,13 +87,11 @@ def transfer(
 
     for recipient_keys, r_files in files_by_recipient.items():
         emails = [get_recipient_email(k) for k in recipient_keys]
-        if needs_argument(protocol, "recipients"):
-            protocol_args["recipients"] = emails
-        if needs_argument(protocol, "two_factor_callback"):
-            protocol_args["two_factor_callback"] = two_factor_callback
-        if needs_argument(protocol, "pkey_password_encoding"):
-            protocol_args["pkey_password_encoding"] = config.ssh_password_encoding
+        if hasattr(protocol, "recipients"):
+            setattr(protocol, "recipients", emails)
+        if hasattr(protocol, "pkey_password_encoding"):
+            setattr(protocol, "pkey_password_encoding", config.ssh_password_encoding)
         with logger.log_task(
             "Transferring files encrypted for " f"{', '.join(emails)}"
         ):
-            protocol(r_files, progress=progress, **protocol_args)
+            protocol.upload(r_files, progress=progress)
